@@ -2,17 +2,17 @@ from __future__ import annotations
 
 import mmap
 import threading
-from contextlib import nullcontext
 from enum import Enum
 from functools import lru_cache
 from itertools import product
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
-    ContextManager,
     Optional,
     Sequence,
     Set,
+    Sized,
+    SupportsInt,
     Union,
     cast,
     overload,
@@ -49,9 +49,30 @@ class ND2File:
     _memmap: mmap.mmap
     _is_legacy: bool
 
-    def __init__(self, path: Union[Path, str]) -> None:
+    def __init__(
+        self,
+        path: Union[Path, str],
+        validate_frames: bool = False,
+        search_window: int = 100,
+    ) -> None:
+        """Open an nd2 file.
+
+        Parameters
+        ----------
+        path : Union[Path, str]
+            Filename of an nd2 file.
+        validate_frames : bool, optional
+            Whether to verify (and attempt to fix) frames whose positions have been
+            shifted relative to the predicted offset (i.e. in a corrupted file),
+            by default False.
+        search_window : int, optional
+            When validate_frames is true, this is the search window (in KB) that will
+            be used to try to find the actual chunk position. by default 100 KB
+        """
         self._path = str(path)
-        self._rdr = get_reader(self._path)
+        self._rdr = get_reader(
+            self._path, validate_frames=validate_frames, search_window=search_window
+        )
         self._closed = False
         self._is_legacy = "Legacy" in type(self._rdr).__name__
         self._lock = threading.RLock()
@@ -234,16 +255,18 @@ class ND2File:
             if isinstance(position, str):
                 try:
                     position = self._position_names().index(position)
-                except ValueError:
-                    raise ValueError(f"{position!r} is not a valid position name")
+                except ValueError as e:
+                    raise ValueError(
+                        f"{position!r} is not a valid position name"
+                    ) from e
             try:
                 pidx = list(self.sizes).index(AXIS.POSITION)
-            except ValueError:
+            except ValueError as exc:
                 if position > 0:
                     raise IndexError(
                         f"Position {position} is out of range. "
                         f"Only 1 position available"
-                    )
+                    ) from exc
                 seqs = range(self._frame_count)
             else:
                 if position >= self.sizes[AXIS.POSITION]:
@@ -260,28 +283,39 @@ class ND2File:
                 seqs = self._seq_index_from_coords(coords)  # type: ignore
                 final_shape[pidx] = 1
 
-        arr = np.stack([self._get_frame(i) for i in seqs])
+        arr: np.ndarray = np.stack([self._get_frame(i) for i in seqs])
         return arr.reshape(final_shape)
 
     def __array__(self) -> np.ndarray:
         """array protocol"""
         return self.asarray()
 
-    def to_dask(self, wrapper=True) -> da.Array:
+    def to_dask(self, wrapper=True, copy=True) -> da.Array:
         """Create dask array (delayed reader) representing image.
 
+        This generally works well, but it remains to be seen whether performance
+        is optimized, or if we're duplicating safety mechanisms. You may try
+        various combinations of `wrapper` and `copy`, setting both to `False`
+        will very likely cause segmentation faults in many cases.  But setting
+        one of them to `False`, may slightly improve read speed in certain
+        cases.
+
         Parameters
         ----------
-        wrapper : bool, optional
+        wrapper : bool
             If True (the default), the returned obect will be a thin subclass of
             a :class:`dask.array.Array` (an
-            `nd2.resource_backed_array.OpeningDaskArray`) that manages the opening
+            `ResourceBackedDaskArray`) that manages the opening
             and closing of this file when getting chunks via compute(). If `wrapper`
             is `False`, then a pure `da.Array` will be returned. However, when that
             array is computed, it will incur a file open/close on *every* chunk
             that is read (in the `_dask_block` method).  As such `wrapper`
             will generally be much faster, however, it *may* fail (i.e. result in
             segmentation faults) with certain dask schedulers.
+        copy : bool
+            If `True` (the default), the dask chunk-reading function will return
+            an array copy. This can avoid segfaults in certain cases, though it
+            may also add overhead.
 
         Returns
         -------
@@ -289,18 +323,17 @@ class ND2File:
         """
         from dask.array import map_blocks
 
-        from .resource_backed_array import ResourceBackedDaskArray
-
         chunks = [(1,) * x for x in self._coord_shape]
         chunks += [(x,) for x in self._frame_shape]
         dask_arr = map_blocks(
             self._dask_block,
-            # the wrapper doesn't need a threading lock
-            nullcontext() if wrapper else self._lock,
+            copy=copy,
             chunks=chunks,
             dtype=self.dtype,
         )
         if wrapper:
+            from resource_backed_dask_array import ResourceBackedDaskArray
+
             # this subtype allows the dask array to re-open the underlying
             # nd2 file on compute.
             return ResourceBackedDaskArray.from_array(dask_arr, self)
@@ -308,16 +341,17 @@ class ND2File:
 
     _NO_IDX = -1
 
-    def _seq_index_from_coords(self, coords: Sequence) -> Union[int, Sequence[int]]:
+    def _seq_index_from_coords(
+        self, coords: Sequence
+    ) -> Union[Sequence[int], SupportsInt]:
         if not self._coord_shape:
             return self._NO_IDX
         return np.ravel_multi_index(coords, self._coord_shape)
 
-    def _dask_block(self, lock: ContextManager, block_id: Tuple[int]) -> np.ndarray:
+    def _dask_block(self, copy: bool, block_id: Tuple[int]) -> np.ndarray:
         if isinstance(block_id, np.ndarray):
             return
-
-        with lock:
+        with self._lock:
             was_closed = self.closed
             if self.closed:
                 self.open()
@@ -332,26 +366,31 @@ class ND2File:
                         )
                     idx = 0
                 data = self._get_frame(cast(int, idx))[(np.newaxis,) * ncoords]
-                # non copy on a closed file WILL segfault.
-                return data.copy() if was_closed else data
+                return data.copy() if copy else data
             finally:
                 if was_closed:
                     self.close()
 
     def to_xarray(
-        self, delayed: bool = True, squeeze: bool = True, position: Optional[int] = None
+        self,
+        delayed: bool = True,
+        squeeze: bool = True,
+        position: Optional[int] = None,
+        copy: bool = True,
     ) -> xr.DataArray:
         """Create labeled xarray representing image.
 
         Parameters
         ----------
-        delayed : bool, optional
+        delayed : bool
             Whether the DataArray should be backed by dask array or numpy array,
             by default True (dask).
-        squeeze : bool, optional
+        squeeze : bool
             Whether to squeeze singleton dimensions, by default True
         position : int, optional
             A specific XY position to extract, by default (None) reads all.
+        copy : bool
+            Only applies when `delayed==True`.  See `to_dask` for details.
 
         Returns
         -------
@@ -360,7 +399,7 @@ class ND2File:
         """
         import xarray as xr
 
-        data = self.to_dask() if delayed else self.asarray(position)
+        data = self.to_dask(copy=copy) if delayed else self.asarray(position)
         dims = list(self.sizes)
         coords = self._expand_coords(squeeze)
         if not squeeze:
@@ -429,7 +468,7 @@ class ND2File:
         """Return a dict that can be used as the coords argument to xr.DataArray"""
         dx, dy, dz = self.voxel_size()
 
-        coords = {
+        coords: Dict[str, Sized] = {
             AXIS.Y: np.arange(self.attributes.heightPx) * dy,
             AXIS.X: np.arange(self.attributes.widthPx) * dx,
             AXIS.CHANNEL: self._channel_names,
